@@ -4,8 +4,6 @@
 
 from __future__ import absolute_import, division, print_function
 
-import warnings
-
 from asn1crypto.algos import DSASignature
 
 import six
@@ -14,31 +12,11 @@ from cryptography import utils
 from cryptography.hazmat.primitives import hashes
 
 
-def decode_rfc6979_signature(signature):
-    warnings.warn(
-        "decode_rfc6979_signature is deprecated and will "
-        "be removed in a future version, use decode_dss_signature instead.",
-        utils.DeprecatedIn10,
-        stacklevel=2
-    )
-    return decode_dss_signature(signature)
-
-
 def decode_dss_signature(signature):
     data = DSASignature.load(signature, strict=True).native
     return data['r'], data['s']
 
 
-def encode_rfc6979_signature(r, s):
-    warnings.warn(
-        "encode_rfc6979_signature is deprecated and will "
-        "be removed in a future version, use encode_dss_signature instead.",
-        utils.DeprecatedIn10,
-        stacklevel=2
-    )
-    return encode_dss_signature(r, s)
-
-
 def encode_dss_signature(r, s):
     if (
         not isinstance(r, six.integer_types) or
